package com.kindlebook.hyrt.chapter12;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.Animation.AnimationListener;
import android.view.animation.AnimationSet;
import android.view.animation.AnimationUtils;
import android.view.animation.DecelerateInterpolator;
import android.view.animation.RotateAnimation;
import android.view.animation.TranslateAnimation;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

public class QuizSplashActivity extends QuizActivity {
    private static final String DEBUG_TAG = "QuizSplashActivity";

    // hold objects for fast access
    private TextView topTitle;
    private TextView bottomTitle;
    private ImageView[] splashImage;
    
    // don't switch away twice
    private boolean alreadySwitched;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.splash);

        try {
            topTitle = (TextView) findViewById(R.id.splashTitleTop);
            bottomTitle = (TextView) findViewById(R.id.splashTitleBottom);

            Animation slide1 = AnimationUtils.loadAnimation(this,
                    R.anim.slide_in_1);
            Animation slide2 = AnimationUtils.loadAnimation(this,
                    R.anim.slide_in_2);

            topTitle.startAnimation(slide1);
            bottomTitle.startAnimation(slide2);

            FrameLayout imagesHolder = (FrameLayout) findViewById(R.id.splashImages);
            int imgCount = imagesHolder.getChildCount();
            splashImage = new ImageView[imgCount];
            for (int i = 0; i < imgCount; i++) {
                splashImage[i] = (ImageView) imagesHolder.getChildAt(i);
                AnimationSet newAnim = createSplashAnim();
                splashImage[i].setAnimation(newAnim);
            }

            alreadySwitched = false;
            slide2.setAnimationListener(new AnimationListener() {
                @Override
                public void onAnimationEnd(Animation animation) {
                    switchToGameMenu();
                }

                @Override
                public void onAnimationRepeat(Animation animation) {
                    // not needed

                }

                @Override
                public void onAnimationStart(Animation animation) {
                    // not needed

                }
            });
        } catch (Exception e) {
            Log.e(DEBUG_TAG, "Animation failed. Perhaps screen is locked?", e);
        }

    }

    public void onScreenTap(View view) {
        switchToGameMenu();
    }
    
    @Override
    protected void onPause() {
        super.onPause();

        // stop animations
        if (topTitle != null) {
            topTitle.clearAnimation();
        }
        if (bottomTitle != null) {
            bottomTitle.clearAnimation();
        }

        if (splashImage != null) {
            for (ImageView img : splashImage) {
                img.clearAnimation();
            }
        }
    }

    private AnimationSet createSplashAnim() {
        // end point offset 150 pixels in either direction
        float targetX = (float) ((Math.random() * 300) - 150);

        // start rotated up to a full circulation left or right
        float startDegrees = (float) (Math.random() * 720) - 360;

        // end plus or minus 20 degrees off vertical
        float endDegrees = (float) (Math.random() * 40) - 20;

        AnimationSet tossIn = new AnimationSet(false);
        TranslateAnimation translateIn = new TranslateAnimation(600, targetX,
                0, 0);
        translateIn.setInterpolator(new DecelerateInterpolator(1.0f));
        RotateAnimation rotate = new RotateAnimation(startDegrees, endDegrees,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f,
                RotateAnimation.RELATIVE_TO_SELF, 0.5f);
        tossIn.addAnimation(translateIn);
        tossIn.addAnimation(rotate);
        tossIn.setDuration(1500);
        tossIn.setFillAfter(true);
        return tossIn;
    }

    private void switchToGameMenu() {
        if (!alreadySwitched) {
            startActivity(new Intent(QuizSplashActivity.this,
                    QuizMenuActivity.class));
            QuizSplashActivity.this.finish();
            alreadySwitched = true;
        }
    }

}