package com.kindlebook.hyrt.chapter13;

import java.io.IOException;
import java.util.Calendar;
import java.util.UUID;
import java.util.Vector;

import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;

import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.app.Dialog;
import android.app.Service;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.text.Editable;
import android.text.TextWatcher;
import android.text.format.DateFormat;
import android.text.format.Time;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

public class QuizSettingsActivity extends QuizActivity {
    private static final String DEBUG_TAG = "QuizSettingsActivity";

    private static final int DATE_DIALOG_ID = 101;
    private static final int PASSWORD_DIALOG_ID = 102;

    private SharedPreferences mGameSettings;
    private EditText nicknameText;
    private EditText emailText;
    private TextView passwordInfo;
    private TextView dobInfo;
    private Spinner spinner;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.settings);

        // Retrieve the shared preferences
        mGameSettings = getSharedPreferences(GAME_PREFERENCES,
                Context.MODE_PRIVATE);
        // Initialize the nickname entry
        initNicknameEntry();
        // Initialize the email entry
        initEmailEntry();
        // Initialize the Password chooser
        initPasswordChooser();
        // Initialize the Date picker
        initDatePicker();
        // Initialize the spinner
        initGenderSpinner();

    }

    @Override
    protected void onPause() {
        String strNickname = nicknameText.getText().toString();
        String strEmail = emailText.getText().toString();

        Editor editor = mGameSettings.edit();
        editor.putString(GAME_PREFERENCES_NICKNAME, strNickname);
        editor.putString(GAME_PREFERENCES_EMAIL, strEmail);

        editor.commit();

        Intent uploadService = new Intent(getApplicationContext(),
                UploaderService.class);
        startService(uploadService);

        super.onPause();
    }

    @Override
    protected Dialog onCreateDialog(int id) {
        switch (id) {
        case DATE_DIALOG_ID: {
            Calendar now = Calendar.getInstance();
            DatePickerDialog dateDialog = new DatePickerDialog(this,
                    new DatePickerDialog.OnDateSetListener() {
                        public void onDateSet(DatePicker view, int year,
                                int monthOfYear, int dayOfMonth) {
                            Time dateOfBirth = new Time();
                            dateOfBirth.set(dayOfMonth, monthOfYear, year);
                            long dtDob = dateOfBirth.toMillis(true);
                            dobInfo.setText(DateFormat.format("MMMM dd, yyyy",
                                    dtDob));
                            Editor editor = mGameSettings.edit();
                            editor.putLong(GAME_PREFERENCES_DOB, dtDob);
                            editor.commit();
                        }
                    }, now.get(Calendar.YEAR), now.get(Calendar.MONTH),
                    now.get(Calendar.DAY_OF_MONTH));
            return dateDialog;
        }
        case PASSWORD_DIALOG_ID: {
            LayoutInflater inflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            final View layout = inflater
                    .inflate(R.layout.password_dialog, null);

            final EditText password = (EditText) layout
                    .findViewById(R.id.passwordEntry);
            final EditText passwordConfirm = (EditText) layout
                    .findViewById(R.id.passwordConfirmEntry);

            final TextView passwordStatus = (TextView) layout
                    .findViewById(R.id.passwordEntryStatus);

            passwordConfirm.addTextChangedListener(new TextWatcher() {
                @Override
                public void afterTextChanged(Editable s) {
                    String passwordString = password.getText().toString();
                    String confirmString = passwordConfirm.getText().toString();
                    if (passwordString.equals(confirmString)) {
                        passwordStatus.setText(R.string.settings_pwd_equal);
                        passwordStatus.setTextColor(getResources().getColor(
                                R.color.good_color));
                    } else {
                        passwordStatus.setText(R.string.settings_pwd_not_equal);
                        passwordStatus.setTextColor(getResources().getColor(
                                R.color.error_color));
                    }
                }

                @Override
                public void beforeTextChanged(CharSequence arg0, int arg1,
                        int arg2, int arg3) {
                    // TODO Auto-generated method stub
                }

                @Override
                public void onTextChanged(CharSequence s, int start,
                        int before, int count) {
                    // TODO Auto-generated method stub
                }

            });

            AlertDialog passwordDialog = new AlertDialog.Builder(this)
                    .setView(layout)
                    .setTitle(R.string.settings_button_pwd)

                    .setPositiveButton(android.R.string.ok,
                            new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog,
                                        int which) {
                                    TextView passwordInfo = (TextView) findViewById(R.id.passwordInfo);
                                    String passwordString = password.getText()
                                            .toString();
                                    String confirmString = passwordConfirm
                                            .getText().toString();
                                    if (passwordString.equals(confirmString)) {
                                        Editor editor = mGameSettings.edit();
                                        editor.putString(
                                                GAME_PREFERENCES_PASSWORD,
                                                passwordString);
                                        editor.commit();
                                        passwordInfo
                                                .setText(R.string.settings_pwd_set);
                                    } else {
                                        Log.w(DEBUG_TAG,
                                                "Warning: Passwords do not match. Not saving. Keeping old password (if set).");
                                    }
                                    QuizSettingsActivity.this
                                            .removeDialog(PASSWORD_DIALOG_ID);
                                }
                            })

                    .setNegativeButton(android.R.string.cancel,
                            new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog,
                                        int whichButton) {
                                    QuizSettingsActivity.this
                                            .removeDialog(PASSWORD_DIALOG_ID);
                                }
                            })

                    .create();
            return passwordDialog;
        }
        }
        return null;
    }

    @Override
    protected void onPrepareDialog(int id, Dialog dialog) {
        super.onPrepareDialog(id, dialog);
        switch (id) {
        case DATE_DIALOG_ID:
            // Handle any DatePickerDialog initialization here
            DatePickerDialog dateDialog = (DatePickerDialog) dialog;
            int day,
            month,
            year;
            // Check for date of birth preference
            if (mGameSettings.contains(GAME_PREFERENCES_DOB)) {
                // Retrieve Birth date setting from preferences
                long msBirthDate = mGameSettings.getLong(GAME_PREFERENCES_DOB,
                        0);
                Time dateOfBirth = new Time();
                dateOfBirth.set(msBirthDate);
                day = dateOfBirth.monthDay;
                month = dateOfBirth.month;
                year = dateOfBirth.year;
            } else {
                Calendar cal = Calendar.getInstance();
                // Today's date fields
                day = cal.get(Calendar.DAY_OF_MONTH);
                month = cal.get(Calendar.MONTH);
                year = cal.get(Calendar.YEAR);
            }
            // Set the date in the DatePicker to the date of birth OR to the
            // current date
            dateDialog.updateDate(year, month, day);
            return;
        }
    }

    /**
     * Initialize the nickname entry
     */
    private void initNicknameEntry() {
        nicknameText = (EditText) findViewById(R.id.nicknameEntry);
        if (mGameSettings.contains(GAME_PREFERENCES_NICKNAME)) {
            nicknameText.setText(mGameSettings.getString(
                    GAME_PREFERENCES_NICKNAME, ""));
        }
    }

    /**
     * Initialize the email entry
     */
    private void initEmailEntry() {
        emailText = (EditText) findViewById(R.id.emailEntry);
        if (mGameSettings.contains(GAME_PREFERENCES_EMAIL)) {
            emailText.setText(mGameSettings.getString(GAME_PREFERENCES_EMAIL,
                    ""));
        }
    }

    /**
     * Initialize the Password chooser
     */
    private void initPasswordChooser() {
        passwordInfo = (TextView) findViewById(R.id.passwordInfo);
        if (mGameSettings.contains(GAME_PREFERENCES_PASSWORD)) {
            passwordInfo.setText(R.string.settings_pwd_set);
        } else {
            passwordInfo.setText(R.string.settings_pwd_not_set);
        }
    }

    /**
     * Called when the user presses the Set Password button
     * 
     * @param view
     *            the button
     */
    public void onSetPasswordButtonClick(View view) {
        showDialog(PASSWORD_DIALOG_ID);
    }

    /**
     * Initialize the Date picker
     */
    private void initDatePicker() {
        dobInfo = (TextView) findViewById(R.id.birthdayInfo);
        if (mGameSettings.contains(GAME_PREFERENCES_DOB)) {
            dobInfo.setText(DateFormat.format("MMMM dd, yyyy",
                    mGameSettings.getLong(GAME_PREFERENCES_DOB, 0)));
        } else {
            dobInfo.setText(R.string.settings_dob_not_set);
        }
    }

    /**
     * Called when the user presses the Pick Date button
     * 
     * @param view
     *            The button
     */
    public void onPickDateButtonClick(View view) {
        showDialog(DATE_DIALOG_ID);
    }

    /**
     * Initialize the spinner
     */
    private void initGenderSpinner() {
        spinner = (Spinner) findViewById(R.id.genderSpinner);
        ArrayAdapter<?> adapter = ArrayAdapter.createFromResource(this,
                R.array.genders, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);
        if (mGameSettings.contains(GAME_PREFERENCES_GENDER)) {
            spinner.setSelection(mGameSettings.getInt(GAME_PREFERENCES_GENDER,
                    0));
        }
        // Handle spinner selections
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            public void onItemSelected(AdapterView<?> parent,
                    View itemSelected, int selectedItemPosition, long selectedId) {
                Editor editor = mGameSettings.edit();
                editor.putInt(GAME_PREFERENCES_GENDER, selectedItemPosition);
                editor.commit();
            }

            public void onNothingSelected(AdapterView<?> parent) {
            }
        });
    }

    @Override
    protected void onDestroy() {
        Log.d(DEBUG_TAG, "SHARED PREFERENCES");
        Log.d(DEBUG_TAG,
                "Nickname is: "
                        + mGameSettings.getString(GAME_PREFERENCES_NICKNAME,
                                "Not set"));
        Log.d(DEBUG_TAG,
                "Email is: "
                        + mGameSettings.getString(GAME_PREFERENCES_EMAIL,
                                "Not set"));
        Log.d(DEBUG_TAG,
                "Gender (M=1, F=2, U=0) is: "
                        + mGameSettings.getInt(GAME_PREFERENCES_GENDER, 0));
        // We are not saving the password yet
        Log.d(DEBUG_TAG,
                "Password is: "
                        + mGameSettings.getString(GAME_PREFERENCES_PASSWORD,
                                "Not set"));
        // We are not saving the date of birth yet
        Log.d(DEBUG_TAG,
                "DOB is: "
                        + DateFormat.format("MMMM dd, yyyy",
                                mGameSettings.getLong(GAME_PREFERENCES_DOB, 0)));
        super.onDestroy();
    }

    public static class UploaderService extends Service {
        private static final String DEBUG_TAG = "QuizSettingsActivity$UploaderService";

        private UploadTask uploader;

        @Override
        public int onStartCommand(Intent intent, int flags, int startId) {
            uploader = new UploadTask();
            uploader.execute();
            Log.d(DEBUG_TAG, "Settings upload requested");
            return START_REDELIVER_INTENT;
        }

        @Override
        public IBinder onBind(Intent intent) {
            // no binding
            return null;
        }

        private class UploadTask extends AsyncTask<Void, String, Boolean> {
            SharedPreferences mGameSettings;

            @Override
            protected void onPostExecute(Boolean result) {
                // perhaps notify the user?
            }

            @Override
            protected void onPreExecute() {
                mGameSettings = getSharedPreferences(GAME_PREFERENCES,
                        Context.MODE_PRIVATE);
            }

            @Override
            protected Boolean doInBackground(Void... params) {
                boolean result = postSettingsToServer();

                Log.d(DEBUG_TAG, "Done uploading settings ");
                return result;
            }

            private boolean postSettingsToServer() {
                boolean succeeded = false;

                // an example of using HttpClient with HTTP GET and form
                // variables

                String uniqueId = mGameSettings.getString(
                        GAME_PREFERENCES_UNIQUE_ID, null);
                Integer playerId = mGameSettings.getInt(
                        GAME_PREFERENCES_PLAYER_ID, -1);
                String nickname = mGameSettings.getString(
                        GAME_PREFERENCES_NICKNAME, "");
                String email = mGameSettings.getString(GAME_PREFERENCES_EMAIL,
                        "");
                String password = mGameSettings.getString(
                        GAME_PREFERENCES_PASSWORD, "");
                Integer score = mGameSettings
                        .getInt(GAME_PREFERENCES_SCORE, -1);
                Integer gender = mGameSettings.getInt(GAME_PREFERENCES_GENDER,
                        -1);
                Long birthdate = mGameSettings.getLong(GAME_PREFERENCES_DOB, 0);

                Vector<NameValuePair> vars = new Vector<NameValuePair>();

                if (uniqueId == null) {
                    // If we don't have a unique id yet, create one and save it
                    uniqueId = UUID.randomUUID().toString();
                    Log.d(DEBUG_TAG, "Unique ID: " + uniqueId);

                    // why not use getDeviceId from TelephonyManager?
                    // See: http://goo.gl/sAbV2
                    // In short, it only works on phones, and even there isn't
                    // necessarily reliable.

                    // save it in the prefs
                    Editor editor = mGameSettings.edit();
                    editor.putString(GAME_PREFERENCES_UNIQUE_ID, uniqueId);
                    editor.commit();
                }

                // add the uniqueId to the request
                vars.add(new BasicNameValuePair("uniqueId", uniqueId));

                if (playerId != -1) {
                    // otherwise, we use the playerId to update data
                    vars.add(new BasicNameValuePair("updateId", playerId
                            .toString()));
                    // and we go ahead and push up the latest score
                    vars.add(new BasicNameValuePair("score", score.toString()));
                }
                vars.add(new BasicNameValuePair("nickname", nickname));
                vars.add(new BasicNameValuePair("email", email));
                vars.add(new BasicNameValuePair("password", password));
                vars.add(new BasicNameValuePair("gender", gender.toString()));
                vars.add(new BasicNameValuePair("dob", birthdate.toString()));

                String url = HYRT_SERVER_ACCOUNT_EDIT + "?"
                        + URLEncodedUtils.format(vars, null);

                HttpGet request = new HttpGet(url);

                try {

                    ResponseHandler<String> responseHandler = new BasicResponseHandler();
                    HttpClient client = new DefaultHttpClient();
                    String responseBody = client.execute(request,
                            responseHandler);

                    if (responseBody != null && responseBody.length() > 0) {
                        Integer resultId = Integer.parseInt(responseBody);
                        Editor editor = mGameSettings.edit();
                        editor.putInt(GAME_PREFERENCES_PLAYER_ID, resultId);
                        editor.commit();
                    }
                    succeeded = true;

                } catch (ClientProtocolException e) {
                    Log.e(DEBUG_TAG, "Failed to get playerId (protocol): ", e);
                } catch (IOException e) {
                    Log.e(DEBUG_TAG, "Failed to get playerId (io): ", e);
                }
                return succeeded;
            }

        }

    }

}